@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: caption.c,v 1.24 2006/10/06 19:24:04 mschimek Exp $ */
+/* $Id: caption.c,v 1.26 2007/11/27 17:39:36 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -81,14 +81,14 @@ language[8] = {
 };
 
 #ifdef __GNUC__
-#define UNUSED __attribute__ ((unused))
+#define UNUSED _vbi_attribute ((unused))
 #else
 #define UNUSED
 #endif
 
 static uint32_t hcrc[128];
 
-static void init_hcrc(void) __attribute__ ((constructor));
+static void init_hcrc(void) _vbi_attribute ((constructor));
 
 
 /*
@@ -1605,3 +1605,10 @@ vbi_fetch_cc_page(vbi_decoder *vbi, vbi_page *pg, vbi_pgno pgno, vbi_bool reset)
 
 	return 1;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
