@@ -45,7 +45,7 @@
 #include "proxy-msg.h"
 #include "proxy-client.h"
 
-#if !defined (__NetBSD__) && !defined (__FreeBSD__)
+#if !defined (__NetBSD__) && !defined (__FreeBSD__) && !defined (__FreeBSD_kernel__)
 #include "videodev.h"
 #include "videodev2k.h"
 # define BASE_VIDIOCPRIVATE      192
@@ -597,3 +597,10 @@ ssize_t read(int fd, void *buf, size_t count)
 }
 
 #endif /* !ENABLE_PROXY || !ENABLE_V4L */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
