@@ -18,10 +18,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: event.h,v 1.11 2006/05/07 20:53:33 mschimek Exp $ */
+/* $Id: event.h,v 1.13 2007/11/27 18:31:06 mschimek Exp $ */
 
 #ifndef EVENT_H
 #define EVENT_H
@@ -716,3 +716,10 @@ extern void		vbi_event_handler_unregister(vbi_decoder *vbi,
 extern void		vbi_send_event(vbi_decoder *vbi, vbi_event *ev);
 
 #endif /* EVENT_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
