@@ -2,7 +2,7 @@
  *  Template for export modules
  */
 
-/* $Id: exp-templ.c,v 1.8 2006/02/10 06:25:37 mschimek Exp $ */
+/* $Id: exp-templ.c,v 1.10 2007/11/27 18:26:32 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -230,15 +230,12 @@ option_set(vbi_export *e, const char *keyword, va_list args)
  *  The output function, mandatory.
  */
 static vbi_bool
-export(vbi_export *e, FILE *fp, vbi_page *pg)
+export(vbi_export *e, vbi_page *pg)
 {
 	tmpl_instance *tmpl = TMPL(e);
 
-	/*
-	 *  Write pg to fp, that's all.
-	 */
-	fp = fp;
-	pg = pg;
+	/* Write pg to target, that's all. */
+	vbi_export_printf (e, "Page %x.%x\n", pg->pgno, pg->subno);
 
 	tmpl->counter++; /* just for fun */
 
@@ -297,3 +294,10 @@ vbi_export_class_tmpl = {
 #if 0
 VBI_AUTOREG_EXPORT_MODULE(vbi_export_class_tmpl)
 #endif
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
