@@ -18,10 +18,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: exp-txt.h,v 1.6 2005/01/19 04:23:53 mschimek Exp $ */
+/* $Id: exp-txt.h,v 1.8 2007/11/27 17:51:11 mschimek Exp $ */
 
 #ifndef EXP_TXT_H
 #define EXP_TXT_H
@@ -62,7 +62,7 @@ extern int		vbi_print_page_region(vbi_page *pg, char *buf, int size,
  * data. Note this function does not append a terminating null
  * character.
  **/
-vbi_inline int
+_vbi_inline int
 vbi_print_page(vbi_page *pg, char *buf, int size,
 	       const char *format, vbi_bool table, vbi_bool ltr)
 {
@@ -75,3 +75,10 @@ vbi_print_page(vbi_page *pg, char *buf, int size,
 /* Private */
 
 #endif /* EXP_TXT_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
