@@ -4,8 +4,9 @@
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: format.h,v 1.7 2003/02/16 21:11:23 mschimek Exp $ */
+/* $Id: format.h,v 1.10 2007/11/27 17:56:03 mschimek Exp $ */
 
 #ifndef FORMAT_H
 #define FORMAT_H
@@ -35,7 +36,7 @@ typedef struct vbi_decoder vbi_decoder;
 
 /**
  * @addtogroup Page Formatted text page
- * @ingroup Service
+ * @ingroup HiDec
  */
 
 /**
@@ -397,3 +398,10 @@ typedef struct vbi_page {
 /* Private */
 
 #endif /* FORMAT_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
