@@ -4,8 +4,9 @@
  *  Copyright (C) 2005 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: idl_demux.h,v 1.5 2006/05/26 00:46:33 mschimek Exp $ */
+/* $Id: idl_demux.h,v 1.8 2007/11/27 17:41:20 mschimek Exp $ */
 
 #ifndef __ZVBI_IDL_DEMUX_H__
 #define __ZVBI_IDL_DEMUX_H__
@@ -25,6 +26,7 @@
 #include <stdio.h>		/* FILE */
 #include <inttypes.h>		/* uint8_t */
 #include "macros.h"
+#include "sliced.h"
 
 VBI_BEGIN_DECLS
 
@@ -47,15 +49,17 @@ VBI_BEGIN_DECLS
  * demultiplexer context.
  */
 typedef struct _vbi_idl_demux vbi_idl_demux;
+/** @} */
 
 /**
+ * @addtogroup IDLDemux Teletext IDL packet demultiplexer
  * @name vbi_idl_demux_cb flags
  * @{
  */
 
 /**
  * Data was lost (not received or uncorrectable) between the current and
- * previous call.
+ * previous vbi_idl_demux_feed() call.
  */
 #define VBI_IDL_DATA_LOST	(1 << 0)
 
@@ -68,6 +72,11 @@ typedef struct _vbi_idl_demux vbi_idl_demux;
 
 /** @} */
 
+/**
+ * @addtogroup IDLDemux Teletext IDL packet demultiplexer
+ * @{
+ */
+
 /**
  * @param dx IDL demultiplexer returned by
  *   vbi_idl_a_demux_new() and given to vbi_idl_demux_feed().
@@ -91,17 +100,35 @@ vbi_idl_demux_cb		(vbi_idl_demux *	dx,
 				 void *			user_data);
 
 extern void
-vbi_idl_demux_reset		(vbi_idl_demux *	dx);
+vbi_idl_demux_reset		(vbi_idl_demux *	dx)
+  _vbi_attribute ((_vbi_nonnull (1)));
 extern vbi_bool
 vbi_idl_demux_feed		(vbi_idl_demux *	dx,
-				 const uint8_t		buffer[42]);
+				 const uint8_t		buffer[42])
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+  _vbi_attribute ((_vbi_nonnull (1, 2)))
+#endif
+  ;
+extern vbi_bool
+vbi_idl_demux_feed_frame	(vbi_idl_demux *	dx,
+				 const vbi_sliced *	sliced,
+				 unsigned int		n_lines)
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+  _vbi_attribute ((_vbi_nonnull (1, 2)))
+#endif
+  ;
 extern void
 vbi_idl_demux_delete		(vbi_idl_demux *	dx);
 extern vbi_idl_demux *
 vbi_idl_a_demux_new		(unsigned int		channel,
 				 unsigned int		address,
 				 vbi_idl_demux_cb *	callback,
-				 void *			user_data);
+				 void *			user_data)
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+  _vbi_attribute ((_vbi_alloc,
+		   _vbi_nonnull (3)))
+#endif
+  ;
 
 /** @} */
 
@@ -117,7 +144,9 @@ vbi_idl_a_demux_new		(unsigned int		channel,
 /** @internal */
 typedef unsigned int _vbi_idl_format;
 
-/** @internal */
+/**
+ * @internal
+ */
 struct _vbi_idl_demux {
 	_vbi_idl_format		format;
 
@@ -138,15 +167,24 @@ struct _vbi_idl_demux {
 };
 
 extern void
-_vbi_idl_demux_destroy		(vbi_idl_demux *	dx);
+_vbi_idl_demux_destroy		(vbi_idl_demux *	dx)
+  _vbi_attribute ((_vbi_nonnull (1)));
 extern vbi_bool
 _vbi_idl_demux_init		(vbi_idl_demux *	dx,
 				 _vbi_idl_format	format,
 				 unsigned int		channel,
 				 unsigned int		address,
 				 vbi_idl_demux_cb *	callback,
-				 void *			user_data);
+				 void *			user_data)
+  _vbi_attribute ((_vbi_nonnull (1, 5)));
 
 VBI_END_DECLS
 
 #endif /* __ZVBI_IDL_DEMUX_H__ */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
