@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,14 @@
 
 #include <sys/select.h>
 #include <sys/ioctl.h>
+
+#ifndef HAVE_S64_U64
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+
 #include "dvb/frontend.h"
 #include "dvb/dmx.h"
 #include "hamm.h"
@@ -82,7 +90,7 @@ static vbi_capture_dvb* dvb_init(const char *dev, char **errstr, int debug)
     }
 
     dvb->debug = debug;
-    dvb->fd = open(dev, O_RDWR | O_NONBLOCK);
+    dvb->fd = open(dev, O_RDONLY | O_NONBLOCK);
     if (-1 == dvb->fd) {
 	asprintf(errstr, _("Cannot open '%s': %d, %s."),
 		 dev, errno, strerror(errno));
@@ -595,3 +603,10 @@ vbi_capture_dvb_new(char *dev, int scanning,
 }
 
 #endif /* !ENABLE_DVB */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
