@@ -4,8 +4,9 @@
  *  Copyright (C) 2004 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: io-sim.h,v 1.6 2006/09/24 03:08:51 mschimek Exp $ */
+/* $Id: io-sim.h,v 1.9 2007/11/27 17:55:50 mschimek Exp $ */
 
 #ifndef __ZVBI_IO_SIM_H__
 #define __ZVBI_IO_SIM_H__
@@ -27,6 +28,11 @@
 #include "sampling_par.h"
 #include "io.h"
 
+#if 3 == VBI_VERSION_MINOR
+#  include "aspect_ratio.h"
+#  include "pdc.h"
+#endif
+
 VBI_BEGIN_DECLS
 
 /* Public */
@@ -47,6 +53,13 @@ vbi_raw_video_image		(uint8_t *		raw,
 				 const vbi_sliced *	sliced,
 				 unsigned int		n_sliced_lines);
 extern vbi_bool
+vbi_raw_add_noise		(uint8_t *		raw,
+				 const vbi_sampling_par *sp,
+				 unsigned int		min_freq,
+				 unsigned int		max_freq,
+				 unsigned int		amplitude,
+				 unsigned int		seed);
+extern vbi_bool
 vbi_raw_vbi_image		(uint8_t *		raw,
 				 unsigned long		raw_size,
 				 const vbi_sampling_par *sp,
@@ -60,6 +73,11 @@ vbi_raw_vbi_image		(uint8_t *		raw,
  * @addtogroup Device
  * @{
  */
+extern void
+vbi_capture_sim_add_noise	(vbi_capture *		cap,
+				 unsigned int		min_freq,
+				 unsigned int		max_freq,
+				 unsigned int		amplitude);
 #if 3 == VBI_VERSION_MINOR
 extern vbi_bool
 vbi_capture_sim_load_vps	(vbi_capture *		cap,
@@ -84,8 +102,19 @@ vbi_capture_sim_new		(int			scanning,
 
 /* Private */
 
+extern unsigned int
+_vbi_capture_sim_get_flags	(vbi_capture *		cap);
+extern void
+_vbi_capture_sim_set_flags	(vbi_capture *		cap,
+				 unsigned int		flags);
+
 #define _VBI_RAW_SWAP_FIELDS	(1 << 0)
 #define _VBI_RAW_SHIFT_CC_CRI	(1 << 1)
+#define _VBI_RAW_LOW_AMP_CC	(1 << 2)
+
+/* NB. Currently this flag has no effect in _vbi_raw_*_image().
+   Call vbi_raw_add_noise() instead. */
+#define _VBI_RAW_NOISE_2	(1 << 17)
 
 extern vbi_bool
 _vbi_raw_video_image		(uint8_t *		raw,
@@ -111,3 +140,10 @@ _vbi_raw_vbi_image		(uint8_t *		raw,
 VBI_END_DECLS
 
 #endif /* __ZVBI_IO_SIM_H__ */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
