@@ -5,8 +5,9 @@
  *  Copyright (C) 2003, 2004 Tom Zoerner
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,11 +16,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 static const char rcsid [] =
-"$Id: io-v4l.c,v 1.33 2006/05/22 09:01:04 mschimek Exp $";
+"$Id: io-v4l.c,v 1.36 2007/11/27 17:42:01 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -62,7 +63,7 @@ static const char rcsid [] =
 /* Custom ioctl of the bttv driver. */
 #define BTTV_VBISIZE		_IOR('v' , BASE_VIDIOCPRIVATE+8, int)
 static __inline__ void IOCTL_ARG_TYPE_CHECK_BTTV_VBISIZE
-  (const int *arg __attribute__ ((unused))) {}
+  (const int *arg _vbi_attribute ((unused))) {}
 
 #undef REQUIRE_SELECT
 #undef REQUIRE_SVBIFMT		/* else accept current parameters */
@@ -1219,7 +1220,7 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 		 *  v4l ioctls, let's see if we can guess the beast.
 		 */
 		printv("Driver doesn't support VIDIOCGCAP\n");
-		strncpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
+		strlcpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
 		v->vcap.name[sizeof(v->vcap.name) - 1] = 0;
 
 		if (!guess_bttv_v4l(v, &strict, given_fd, scanning))
@@ -1228,7 +1229,7 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 		if (v->vcap.name[0] != 0) {
 			printv("Driver name '%s'\n", v->vcap.name);
 		} else {
-			strncpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
+			strlcpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
 			v->vcap.name[sizeof(v->vcap.name) - 1] = 0;
 		}
 
@@ -1439,3 +1440,10 @@ vbi_capture_v4l_new(const char *dev_name, int scanning,
 }
 
 #endif /* !ENABLE_V4L */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
