@@ -5,8 +5,9 @@
  *  Copyright (C) 2003-2004 Tom Zoerner
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,11 +16,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 static const char rcsid [] =
-"$Id: io-v4l2k.c,v 1.41 2006/05/31 03:54:00 mschimek Exp $";
+"$Id: io-v4l2k.c,v 1.48 2007/11/27 17:55:41 mschimek Exp $";
 
 /*
  *  Around Oct-Nov 2002 the V4L2 API was revised for inclusion into
@@ -55,6 +56,14 @@ static const char rcsid [] =
 #include "raw_decoder.h"
 #include "version.h"
 
+#ifndef HAVE_S64_U64
+#  include <inttypes.h>
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+
 #include "videodev2k.h"
 #include "_videodev2k.h"
 
@@ -109,6 +118,7 @@ typedef struct vbi_capture_v4l2 {
 	vbi_bool		bttv_offset_fix;
 	vbi_bool		cx88_ntsc_fix;
 	vbi_bool		bttv_min_start_fix;
+	vbi_bool		bttv_ntsc_rate_fix;
 
 	_vbi_log_hook		log;
 
@@ -229,6 +239,7 @@ v4l2_stream_alloc(vbi_capture_v4l2 *v, char ** errstr)
 
 		vbuf.type = v->btype;
 		vbuf.index = v->num_raw_buffers;
+		vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QUERYBUF, &vbuf)) {
 			asprintf (errstr,
@@ -335,9 +346,10 @@ restart_stream			(vbi_capture_v4l2 *	v)
 
 		vbuf.index = i;
 		vbuf.type = v->btype;
+		vbuf.memory = V4L2_MEMORY_MMAP;
 
-		if (-1 == xioctl (v, VIDIOC_QBUF, &vbuf))
-			return FALSE;
+		/* Error ignored. */
+		xioctl (v, VIDIOC_QBUF, &vbuf);
 	}
 
 	if (-1 == xioctl (v, VIDIOC_STREAMON, &v->btype))
@@ -372,6 +384,7 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 	} else if (ENQUEUE_IS_UNQUEUED(v->enqueue)) {
 		v->vbuf.type = v->btype;
 		v->vbuf.index = v->enqueue;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QBUF, &v->vbuf)) {
 			error (&v->log,
@@ -398,6 +411,7 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 		}
 
 		v->vbuf.type = v->btype;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		/* retrieve the captured frame from the queue */
 		r = xioctl (v, VIDIOC_DQBUF, &v->vbuf);
@@ -410,10 +424,9 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 			       "Failed to dequeue buffer, errno %d.",
 			       errno);
 
-			/* On EIO bttv dequeues the buffer, or it does not,
-			   or it resets the hardware (SCERR).  QBUF alone
-			   is insufficient.
-			   Actually the caller should restart on error. */
+			/* On EIO bttv dequeues the buffer, other drivers
+			   may not. Actually the caller should restart
+			   on error. */
 
 			/* Errors ignored. */
 			restart_stream (v);
@@ -501,6 +514,7 @@ v4l2_stream_flush(vbi_capture *vc)
 	if (ENQUEUE_IS_UNQUEUED(v->enqueue)) {
 		v->vbuf.type = v->btype;
 		v->vbuf.index = v->enqueue;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QBUF, &v->vbuf)) {
 			error (&v->log,
@@ -1057,11 +1071,12 @@ v4l2_update_services(vbi_capture *vc,
 		    && 9 == vfmt.fmt.vbi.start[0]
 		    && 272 == vfmt.fmt.vbi.start[1]) {
 			/* Captures only 288 * 4 samples/line,
-			   work-around not possible. */
+			   no work-around possible. */
+			/* XXX when was this fixed? */
 			asprintf (errstr,
 				  _("A known bug in driver %s %u.%u.%u "
 				    "impedes VBI capturing in NTSC mode. "
-				    "Please upgrade."),
+				    "Please upgrade the driver."),
 				  v->vcap.driver,
 				  (v->vcap.version >> 16) & 0xFF,
 				  (v->vcap.version >> 8) & 0xFF,
@@ -1083,6 +1098,13 @@ v4l2_update_services(vbi_capture *vc,
 			fixed = TRUE;
 		}
 
+		if (v->bttv_ntsc_rate_fix
+		    && 525 == v->sp.scanning
+		    && 35468950 == vfmt.fmt.vbi.sampling_rate) {
+			vfmt.fmt.vbi.sampling_rate = 28636363;
+			fixed = TRUE;
+		}
+
 		if (fixed)
 			print_vfmt (v, "Fixes applied: ", &vfmt);
 	}
@@ -1391,6 +1413,7 @@ vbi_capture_v4l2k_new		(const char *		dev_name,
 			v->bttv_min_start_fix = TRUE;
 		}
 		v->bttv_offset_fix = TRUE;
+		v->bttv_ntsc_rate_fix = TRUE;
 	} else if (0 == strcmp ((char *) v->vcap.driver, "saa7134")) {
 		if (v->vcap.version <= 0x00020C)
 			v->saa7134_ntsc_fix = TRUE;
@@ -1521,3 +1544,10 @@ vbi_capture_v4l2k_new(const char *dev_name, int fd, int buffers,
 }
 
 #endif /* !ENABLE_V4L2 */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
