@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: io.c,v 1.15 2006/05/31 03:52:08 mschimek Exp $ */
+/* $Id: io.c,v 1.18 2007/11/27 18:31:06 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -750,16 +750,16 @@ device_ioctl			(FILE *			fp,
 		fputc ('(', fp);
       
 		if (IOCTL_WRITE (cmd))
-			log_fn (fp, cmd, IOCTL_READ (cmd) ? 2 : 0, &buf);
+			log_fn (fp, cmd, IOCTL_READ (cmd) ? 3 : 2, &buf);
 
 		if (-1 == err) {
 			fprintf (fp, "), errno = %d, %s\n",
-				 errno, strerror (errno));
+				 saved_errno, strerror (saved_errno));
 		} else {
 			if (IOCTL_READ (cmd)) {
 				fputs (") -> (", fp);
 				log_fn (fp, cmd, IOCTL_WRITE (cmd) ?
-					1 : 0, arg);
+					3 : 1, arg);
 			}
 
 			fputs (")\n", fp);
@@ -853,3 +853,10 @@ device_munmap			(FILE *			fp,
 
 	return r;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
