@@ -14,15 +14,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: io.h,v 1.20 2006/05/24 04:46:12 mschimek Exp $ */
+/* $Id: io.h,v 1.23 2007/11/27 18:31:06 mschimek Exp $ */
 
 #ifndef IO_H
 #define IO_H
 
 #include "decoder.h"
+#include "bit_slicer.h"
 
 /* Public */
 
@@ -241,6 +242,14 @@ struct vbi_capture {
 					 vbi_capture_buffer **,
 					 vbi_capture_buffer **,
 					 const struct timeval *);
+	vbi_bool		(* sampling_point)
+					(vbi_capture *,
+					 vbi3_bit_slicer_point *,
+					 unsigned int row,
+					 unsigned int nth_bit);
+	vbi_bool		(* debug)
+					(vbi_capture *,
+					 vbi_bool enable);
 	vbi_raw_decoder *	(* parameters)(vbi_capture *);
         unsigned int            (* update_services)(vbi_capture *,
                                          vbi_bool, vbi_bool,
@@ -264,3 +273,10 @@ vbi_capture_io_select		(int			fd,
 				 struct timeval *	timeout);
 
 #endif /* IO_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
