@@ -4,8 +4,9 @@
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: lang.c,v 1.6 2005/01/20 01:38:56 mschimek Exp $ */
+/* $Id: lang.c,v 1.11 2007/11/27 17:55:35 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -175,7 +176,7 @@ gfx_transcript[] = {
 /*
  *  Teletext character set
  *
- *  (This is somehow related to ISO 6937 -
+ *  (Similar to ISO 6937 -
  *   ftp://dkuug.dk/i18n/charmaps/ISO_6937)
  */
 
@@ -477,7 +478,7 @@ vbi_teletext_unicode(vbi_character_set s, vbi_national_subset n, unsigned int c)
 
 	default:
 		fprintf(stderr, "%s: unknown char set %d\n",
-			__PRETTY_FUNCTION__, s);
+			__FUNCTION__, s);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -533,51 +534,6 @@ vbi_teletext_composed_unicode(unsigned int a, unsigned int c)
 	return 0;
 }
 
-/*
- *  Closed Caption character set
- *
- *  EIA 608
- *
- *  (from Video Demystified, Table 8.31. Closed Captioning Basic Character Set)
- */
-static const unsigned short
-caption[96] = {
-  0x0020u, 0x0021u, 0x0022u, 0x0023u, 0x0024u, 0x0025u, 0x0026u, 0x0027u, 0x0028u, 0x0029u, 0x00E1u, 0x002Bu, 0x002Cu, 0x002Du, 0x002Eu, 0x002Fu,
-  0x0030u, 0x0031u, 0x0032u, 0x0033u, 0x0034u, 0x0035u, 0x0036u, 0x0037u, 0x0038u, 0x0039u, 0x003Au, 0x003Bu, 0x003Cu, 0x003Du, 0x003Eu, 0x003Fu,
-  0x0040u, 0x0041u, 0x0042u, 0x0043u, 0x0044u, 0x0045u, 0x0046u, 0x0047u, 0x0048u, 0x0049u, 0x004Au, 0x004Bu, 0x004Cu, 0x004Du, 0x004Eu, 0x004Fu,
-  0x0050u, 0x0051u, 0x0052u, 0x0053u, 0x0054u, 0x0055u, 0x0056u, 0x0057u, 0x0058u, 0x0059u, 0x005Au, 0x005Bu, 0x00E9u, 0x005Du, 0x00EDu, 0x00F3u,
-  0x00FAu, 0x0061u, 0x0062u, 0x0063u, 0x0064u, 0x0065u, 0x0066u, 0x0067u, 0x0068u, 0x0069u, 0x006Au, 0x006Bu, 0x006Cu, 0x006Du, 0x006Eu, 0x006Fu,
-  0x0070u, 0x0071u, 0x0072u, 0x0073u, 0x0074u, 0x0075u, 0x0076u, 0x0077u, 0x0078u, 0x0079u, 0x007Au, 0x00E7u, 0x00F7u, 0x00D1u, 0x00F1u, 0x25A0u
-};
-
-static const unsigned short
-caption_special[] = {
-  0x00AEu, 0x00B0u, 0x00BDu, 0x00BFu, 0x2122u, 0x00A2u, 0x00A3u, 0x266Au,
-  0x00E0u, 0x0020u, 0x00E8u, 0x00E2u, 0x00EAu, 0x00EEu, 0x00F4u, 0x00FBu
-};
-
-/**
- * @internal
- * @param c Character code in range 0x00 ... 0x0F and 0x20 ... 0x7F.
- * 
- * Translate Closed Caption character code to Unicode. Codes
- * in range 0x00 ... 0x0F are Special Characters (Closed
- * Caption commands 0x22 0x3x, 0x32 0x3x).
- *
- * @return
- * Unicode value.
- */
-unsigned int
-vbi_caption_unicode(unsigned int c)
-{
-	assert(c <= 15 || (c >= 0x20 && c <= 0x7F));
-
-	if (c < 0x10)
-		return caption_special[c];
-	else
-		return caption[c - 0x20];
-}
-
 static inline int
 is_blank(vbi_char c)
 {
@@ -662,3 +618,262 @@ vbi_optimize_page(vbi_page *pg, int column, int row, int width, int height)
 			*cp = l = c;
 		}
 }
+
+/*
+	Closed Caption character set
+*/
+
+/* Closed Caption Basic Character Set.
+   47 CFR Section 15.119 (g) */
+static const uint16_t
+caption [96][2] = {
+	{ 0x0020, 0x0020 },
+	{ 0x0021, 0x0021 },
+	{ 0x0022, 0x0022 },
+	{ 0x0023, 0x0023 },
+	{ 0x0024, 0x0024 },
+	{ 0x0025, 0x0025 },
+	{ 0x0026, 0x0026 },
+	{ 0x0027, 0x0027 },
+	{ 0x0028, 0x0028 },
+	{ 0x0029, 0x0029 },
+	{ 0x00E1, 0x00C1 }, /* 0x2A a with acute accent */
+	{ 0x002B, 0x002B },
+	{ 0x002C, 0x002C },
+	{ 0x002D, 0x002D },
+	{ 0x002E, 0x002E },
+	{ 0x002F, 0x002F },
+	{ 0x0030, 0x0030 },
+	{ 0x0031, 0x0031 },
+	{ 0x0032, 0x0032 },
+	{ 0x0033, 0x0033 },
+	{ 0x0034, 0x0034 },
+	{ 0x0035, 0x0035 },
+	{ 0x0036, 0x0036 },
+	{ 0x0037, 0x0037 },
+	{ 0x0038, 0x0038 },
+	{ 0x0039, 0x0039 },
+	{ 0x003A, 0x003A },
+	{ 0x003B, 0x003B },
+	{ 0x003C, 0x003C },
+	{ 0x003D, 0x003D },
+	{ 0x003E, 0x003E },
+	{ 0x003F, 0x003F },
+	{ 0x0040, 0x0040 },
+	{ 0x0041, 0x0041 },
+	{ 0x0042, 0x0042 },
+	{ 0x0043, 0x0043 },
+	{ 0x0044, 0x0044 },
+	{ 0x0045, 0x0045 },
+	{ 0x0046, 0x0046 },
+	{ 0x0047, 0x0047 },
+	{ 0x0048, 0x0048 },
+	{ 0x0049, 0x0049 },
+	{ 0x004A, 0x004A },
+	{ 0x004B, 0x004B },
+	{ 0x004C, 0x004C },
+	{ 0x004D, 0x004D },
+	{ 0x004E, 0x004E },
+	{ 0x004F, 0x004F },
+	{ 0x0050, 0x0050 },
+	{ 0x0051, 0x0051 },
+	{ 0x0052, 0x0052 },
+	{ 0x0053, 0x0053 },
+	{ 0x0054, 0x0054 },
+	{ 0x0055, 0x0055 },
+	{ 0x0056, 0x0056 },
+	{ 0x0057, 0x0057 },
+	{ 0x0058, 0x0058 },
+	{ 0x0059, 0x0059 },
+	{ 0x005A, 0x005A },
+	{ 0x005B, 0x005B },
+	{ 0x00E9, 0x00C9 }, /* 0x5C e with acute accent */
+	{ 0x005D, 0x005D },
+	{ 0x00ED, 0x00CD }, /* 0x5E i with acute accent */
+	{ 0x00F3, 0x00D3 }, /* 0x5F o with acute accent */
+	{ 0x00FA, 0x00DA }, /* 0x60 u with acute accent */
+	{ 0x0061, 0x0041 },
+	{ 0x0062, 0x0042 },
+	{ 0x0063, 0x0043 },
+	{ 0x0064, 0x0044 },
+	{ 0x0065, 0x0045 },
+	{ 0x0066, 0x0046 },
+	{ 0x0067, 0x0047 },
+	{ 0x0068, 0x0048 },
+	{ 0x0069, 0x0049 },
+	{ 0x006A, 0x004A },
+	{ 0x006B, 0x004B },
+	{ 0x006C, 0x004C },
+	{ 0x006D, 0x004D },
+	{ 0x006E, 0x004E },
+	{ 0x006F, 0x004F },
+	{ 0x0070, 0x0050 },
+	{ 0x0071, 0x0051 },
+	{ 0x0072, 0x0052 },
+	{ 0x0073, 0x0053 },
+	{ 0x0074, 0x0054 },
+	{ 0x0075, 0x0055 },
+	{ 0x0076, 0x0056 },
+	{ 0x0077, 0x0057 },
+	{ 0x0078, 0x0058 },
+	{ 0x0079, 0x0059 },
+	{ 0x007A, 0x005A },
+	{ 0x00E7, 0x00C7 }, /* 0x7B c with cedilla */
+	{ 0x00F7, 0x00F7 }, /* 0x7C Division sign */
+	{ 0x00D1, 0x00D1 }, /* 0x7D N with tilde */
+	{ 0x00F1, 0x00D1 }, /* 0x7E n with tilde */
+	{ 0x25A0, 0x25A0 }  /* 0x7F Black square */
+};
+
+/* Closed Caption Special Characters.
+   47 CFR Section 15.119 (g) */
+static const uint16_t
+caption_special [16][2] = {
+	{ 0x00AE, 0x00AE }, /* 0x1130 Registered symbol */
+	{ 0x00B0, 0x00B0 }, /* 0x1131 Degree sign */
+	{ 0x00BD, 0x00BD }, /* 0x1132 1/2 */
+	{ 0x00BF, 0x00BF }, /* 0x1133 Inverse question mark */
+	{ 0x2122, 0x2122 }, /* 0x1134 Trademark symbol */
+	{ 0x00A2, 0x00A2 }, /* 0x1135 Cents sign */
+	{ 0x00A3, 0x00A3 }, /* 0x1136 Pounds sign */
+	{ 0x266A, 0x266A }, /* 0x1137 Music note */
+	{ 0x00E0, 0x00C0 }, /* 0x1138 a with grave accent */
+	{ 0x0020, 0x0020 }, /* 0x1139 Transparent space */
+	{ 0x00E8, 0x00C8 }, /* 0x113A e with grave accent */
+	{ 0x00E2, 0x00C2 }, /* 0x113B a with circumflex */
+	{ 0x00EA, 0x00CA }, /* 0x113C e with circumflex */
+	{ 0x00EE, 0x00CE }, /* 0x113D i with circumflex */
+	{ 0x00F4, 0x00D4 }, /* 0x113E o with circumflex */
+	{ 0x00FB, 0x00DB }  /* 0x113F u with circumflex */
+};
+
+/* Closed Caption Extended Character Set.
+   EIA 608-B Section 6.4.2 */
+static const uint16_t
+caption_extended2 [32][2] = {
+	{ 0x00C1, 0x00C1 }, /* 0x1220 A with acute accent */
+	{ 0x00C9, 0x00C9 }, /* 0x1221 E with acute accent */
+	{ 0x00D3, 0x00D3 }, /* 0x1222 O with acute accent */
+	{ 0x00DA, 0x00DA }, /* 0x1223 U with acute accent */
+	{ 0x00DC, 0x00DC }, /* 0x1224 U with diaeresis */
+	{ 0x00FC, 0x00DC }, /* 0x1225 u with diaeresis */
+	{ 0x2018, 0x2018 }, /* 0x1226 Opening single quote */
+	{ 0x00A1, 0x00A1 }, /* 0x1227 Inverted exclamation mark */
+	{ 0x002A, 0x002A }, /* 0x1228 Asterisk */
+	{ 0x0027, 0x0027 }, /* 0x1229 Plain single quote */
+	{ 0x2500, 0x2500 }, /* 0x122A Em dash (for box drawing) */
+	{ 0x00A9, 0x00A9 }, /* 0x122B Copyright */
+	{ 0x2120, 0x2120 }, /* 0x122C Service mark */
+	{ 0x2022, 0x2022 }, /* 0x122D Round bullet */
+	{ 0x201C, 0x201C }, /* 0x122E Opening double quotes */
+	{ 0x201D, 0x201D }, /* 0x122F Closing double quotes */
+	{ 0x00C0, 0x00C0 }, /* 0x1230 A with grave accent */
+	{ 0x00C2, 0x00C2 }, /* 0x1231 A with circumflex accent */
+	{ 0x00C7, 0x00C7 }, /* 0x1232 C with cedilla */
+	{ 0x00C8, 0x00C8 }, /* 0x1233 E with grave accent */
+	{ 0x00CA, 0x00CA }, /* 0x1234 E with circumflex accent */
+	{ 0x00CB, 0x00CB }, /* 0x1235 E with diaeresis */
+	{ 0x00EB, 0x00CB }, /* 0x1236 e with diaeresis */
+	{ 0x00CE, 0x00CE }, /* 0x1237 I with circumflex */
+	{ 0x00CF, 0x00CF }, /* 0x1238 I with diaeresis */
+	{ 0x00EE, 0x00CE }, /* 0x1239 i with diaeresis */
+	{ 0x00D4, 0x00D4 }, /* 0x123A O with circumflex */
+	{ 0x00D9, 0x00D9 }, /* 0x123B U with grave accent */
+	{ 0x00F9, 0x00D9 }, /* 0x123C u with grave accent */
+	{ 0x00DB, 0x00DB }, /* 0x123D U with circumflex accent */
+	{ 0x00AB, 0x00AB }, /* 0x123E Opening guillemets */
+	{ 0x00BB, 0x00BB }  /* 0x123F Closing guillemets */
+};
+
+/* Closed Caption Extended Character Set.
+   EIA 608-B Section 6.4.2 */
+static const uint16_t
+caption_extended3 [32][2] = {
+	{ 0x00C3, 0x00C3 }, /* 0x1320 A with tilde */
+	{ 0x00E3, 0x00C3 }, /* 0x1321 a with tilde */
+	{ 0x00CD, 0x00CD }, /* 0x1322 I with acute accent */
+	{ 0x00CC, 0x00CC }, /* 0x1323 I with grave accent */
+	{ 0x00EC, 0x00CC }, /* 0x1324 i with grave accent */
+	{ 0x00D2, 0x00D2 }, /* 0x1325 O with grave accent */
+	{ 0x00F2, 0x00D2 }, /* 0x1326 o with grave accent */
+	{ 0x00D5, 0x00D5 }, /* 0x1327 O with tilde */
+	{ 0x00F5, 0x00D5 }, /* 0x1328 o with tilde */
+	{ 0x007B, 0x007B }, /* 0x1329 Opening curly brace */
+	{ 0x007D, 0x007D }, /* 0x132A Closing curly brace */
+	{ 0x005C, 0x005C }, /* 0x132B Backslash */
+	{ 0x005E, 0x005E }, /* 0x132C Caret */
+	{ 0x005F, 0x005F }, /* 0x132D Underscore */
+	{ 0x007C, 0x007C }, /* 0x132E Vertical bar */
+	{ 0x007E, 0x007E }, /* 0x132F Tilde */
+	{ 0x00C4, 0x00C4 }, /* 0x1330 A with diaeresis */
+	{ 0x00E4, 0x00C4 }, /* 0x1331 a with diaeresis */
+	{ 0x00D6, 0x00D6 }, /* 0x1332 O with diaeresis */
+	{ 0x00F6, 0x00D6 }, /* 0x1333 o with diaeresis */
+	{ 0x00DF, 0x00DF }, /* 0x1334 Sharp s */
+	{ 0x00A5, 0x00A5 }, /* 0x1335 Yen sign */
+	{ 0x00A4, 0x00A4 }, /* 0x1336 Currency sign */
+	{ 0x2502, 0x2502 }, /* 0x1337 Vertical bar (for box drawing) */
+	{ 0x00C5, 0x00C5 }, /* 0x1338 A with ring above */
+	{ 0x00E5, 0x00C5 }, /* 0x1339 a with ring above */
+	{ 0x00D8, 0x00D8 }, /* 0x133A O with slash */
+	{ 0x00F8, 0x00D8 }, /* 0x133B o with slash */
+	{ 0x250C, 0x250C }, /* 0x133C Upper left corner */
+	{ 0x2510, 0x2510 }, /* 0x133D Upper right corner */
+	{ 0x2514, 0x2514 }, /* 0x133E Lower left corner */
+	{ 0x2518, 0x2518 }  /* 0x133F Lower right corner */
+};
+
+/**
+ * @ingroup Conv
+ * @param c Character code in range 0x20 ... 0x7F,
+ *   0x1130 ... 0x113F, 0x1930 ... 0x193F, 0x1220 ... 0x123F,
+ *   0x1A20 ... 0x1A3F, 0x1320 ... 0x133F, 0x1B20 ... 0x1B3F.
+ * @param to_upper Convert the character to upper case. (Often
+ *   programs are captioned in all upper case, but except for
+ *   one character the basic and special CC character sets contain
+ *   only lower case accented characters.)
+ *
+ * Converts a Closed Caption character code to Unicode. Codes
+ * in range 0x1130 to 0x1B3F are "special characters" and "extended
+ * characters" (e.g. caption command 11 37).
+ *
+ * @see vbi_strndup_iconv_caption()
+ *
+ * @return
+ * Unicode value or 0 if @a c is outside the valid ranges.
+ *
+ * @since 0.2.23
+ */
+unsigned int
+vbi_caption_unicode		(unsigned int		c,
+				 vbi_bool		to_upper)
+{
+	to_upper = !!to_upper;
+
+	if (likely (c < 0x80)) {
+		if (likely (c >= 0x20)) {
+			return caption[c - 0x20][to_upper];
+		}
+	} else {
+		c &= ~0x0700;
+
+		if (c < 0x1240) {
+			if (c < 0x1140 && c >= 0x1130) {
+				return caption_special[c - 0x1130][to_upper];
+			} else if (c >= 0x1220) {
+				return caption_extended2[c - 0x1220][to_upper];
+			}
+		} else if (c < 0x1340 && c >= 0x1320) {
+			return caption_extended3[c - 0x1320][to_upper];
+		}
+	}
+
+	return 0;
+}
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
