@@ -4,8 +4,9 @@
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: lang.h,v 1.6 2006/10/06 19:23:24 mschimek Exp $ */
+/* $Id: lang.h,v 1.8 2007/11/27 17:50:54 mschimek Exp $ */
 
 #ifndef LANG_H
 #define LANG_H
@@ -108,7 +109,7 @@ typedef struct vbi_font_descr vbi_font_descr;
  * Mosaics and Line Drawing Set, with codes U+EE00 ... U+EFFF, and
  * Teletext DRCS coded U+F000 ... U+F7FF.
  */
-vbi_inline vbi_bool
+_vbi_inline vbi_bool
 vbi_is_print(unsigned int unicode)
 {
 	return unicode < 0xE600;
@@ -122,7 +123,7 @@ vbi_is_print(unsigned int unicode)
  * @c TRUE if @a unicode represents a Teletext G1 Block Mosaic or G3 Smooth
  * Mosaics and Line Drawing Set, that is a code in range U+EE00 ... U+EFFF.
  */
-vbi_inline vbi_bool
+_vbi_inline vbi_bool
 vbi_is_gfx(unsigned int unicode)
 {
 	return unicode >= 0xEE00 && unicode <= 0xEFFF;
@@ -136,7 +137,7 @@ vbi_is_gfx(unsigned int unicode)
  * @c TRUE if @a unicode represents a Teletext DRCS (Dynamically
  * Redefinable Character), that is a code in range U+F000 ... U+F7FF.
  **/
-vbi_inline vbi_bool
+_vbi_inline vbi_bool
 vbi_is_drcs(unsigned int unicode)
 {
 	return unicode >= 0xF000;
@@ -153,3 +154,10 @@ extern unsigned int	vbi_teletext_composed_unicode(unsigned int a, unsigned int c
 extern void		vbi_optimize_page(vbi_page *pg, int column, int row, int width, int height);
 
 #endif /* LANG_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
