@@ -1,7 +1,7 @@
 /*
  *  libzvbi - Miscellaneous cows and chickens
  *
- *  Copyright (C) 2002-2006 Michael H. Schimek
+ *  Copyright (C) 2002-2007 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: misc.h,v 1.15 2006/05/26 00:44:34 mschimek Exp $ */
+/* $Id: misc.h,v 1.19 2007/11/27 17:41:50 mschimek Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -28,10 +28,13 @@
 #include <stddef.h>
 #include <stdarg.h>
 #include <string.h>
-#include <inttypes.h>
+#include <inttypes.h>		/* (u)intXX_t */
+#include <sys/types.h>		/* (s)size_t */
+#include <limits.h>		/* (S)SIZE_MAX */
 #include <assert.h>
 
 #include "macros.h"
+#include "version.h"
 
 #define N_ELEMENTS(array) (sizeof (array) / sizeof (*(array)))
 
@@ -84,7 +87,7 @@
 		_t >>= sizeof (_t) * 8 - 1;				\
 		_n ^= _t;						\
 		_n -= _t;						\
-	} else if (_n < 0) { /* also warns if n is unsigned */		\
+	} else if (_n < 0) { /* also warns if n is unsigned type */	\
 		_n = -_n;						\
 	}								\
 	/* return */ _n;						\
@@ -196,16 +199,32 @@ do {									\
 	 + (((m) & 0xFF00) << 8)					\
 	 + (((m) & 0xFF) << 24))
 
+#ifdef HAVE_BUILTIN_POPCOUNT
+#  define popcnt(x) __builtin_popcount ((uint32_t)(x))
+#else
+#  define popcnt(x) _vbi_popcnt (x)
+#endif
+
+extern unsigned int
+_vbi_popcnt			(uint32_t		x);
+
 /* NB GCC inlines and optimizes these functions when size is const. */
 #define SET(var) memset (&(var), ~0, sizeof (var))
 
 #define CLEAR(var) memset (&(var), 0, sizeof (var))
 
-#define COPY(d, s) /* useful to copy arrays, otherwise use assignment */ \
+/* Useful to copy arrays, otherwise use assignment. */
+#define COPY(d, s)							\
 	(assert (sizeof (d) == sizeof (s)), memcpy (d, s, sizeof (d)))
 
-/* Copy a string const, returns FALSE if not NUL terminated. */
-#define STRCOPY(d, s) (strlcpy (d, s, sizeof (d)) < sizeof (d))
+/* Copy string const into char array. */
+#define STRACPY(array, s)						\
+do {									\
+	/* Complain if s is no string const or won't fit. */		\
+	const char t_[sizeof (array) - 1] _vbi_attribute ((unused)) = s; \
+									\
+	memcpy (array, s, sizeof (s));					\
+} while (0)
 
 /* Copy bits through mask. */
 #define COPY_SET_MASK(dest, from, mask)					\
@@ -219,17 +238,31 @@ do {									\
 #define COPY_SET_CLEAR(dest, set, clear)				\
 	(dest = (dest & ~(clear)) | (set))
 
-/* For debugging. */
-#define vbi_malloc malloc
-#define vbi_realloc realloc
-#define vbi_strdup strdup
-#define vbi_free free
-#define vbi_cache_malloc malloc
-#define vbi_cache_free free
+/* For applications, debugging and fault injection during unit tests. */
+
+#if 2 == VBI_VERSION_MINOR
+#  define vbi_malloc malloc
+#  define vbi_realloc realloc
+#  define vbi_strdup strdup
+#  define vbi_free free
+#else
+extern void *
+(* vbi_malloc)			(size_t);
+extern void *
+(* vbi_realloc)			(void *,
+				 size_t);
+extern char *
+(* vbi_strdup)			(const char *);
+extern void
+(* vbi_free)			(void *);
+#endif
+
+#define vbi_cache_malloc vbi_malloc
+#define vbi_cache_free vbi_free
 
 /* Helper functions. */
 
-vbi_inline int
+_vbi_inline int
 _vbi_to_ascii			(int			c)
 {
 	if (c < 0)
@@ -254,6 +287,19 @@ _vbi_keyword_lookup		(int *			value,
 				 const _vbi_key_value_pair * table,
 				 unsigned int		n_pairs);
 
+extern void
+_vbi_shrink_vector_capacity	(void **		vector,
+				 size_t *		capacity,
+				 size_t			min_capacity,
+				 size_t			element_size)
+  _vbi_attribute ((_vbi_nonnull (1, 2)));
+extern vbi_bool
+_vbi_grow_vector_capacity	(void **		vector,
+				 size_t *		capacity,
+				 size_t			min_capacity,
+				 size_t			element_size)
+  _vbi_attribute ((_vbi_nonnull (1, 2)));
+
 /* Logging stuff. */
 
 extern _vbi_log_hook		_vbi_global_log;
@@ -262,6 +308,7 @@ extern void
 _vbi_log_vprintf		(vbi_log_fn		log_fn,
 				 void *			user_data,
 				 vbi_log_mask		level,
+				 const char *		source_file,
 				 const char *		context,
 				 const char *		templ,
 				 va_list		ap);
@@ -269,6 +316,7 @@ extern void
 _vbi_log_printf			(vbi_log_fn		log_fn,
 				 void *			user_data,
 				 vbi_log_mask		level,
+				 const char *		source_file,
 				 const char *		context,
 				 const char *		templ,
 				 ...);
@@ -280,10 +328,21 @@ do {									\
 	if ((NULL != _h && 0 != (_h->mask & level))			\
 	    || (_h = &_vbi_global_log, 0 != (_h->mask & level)))	\
 		_vbi_log_printf (_h->fn, _h->user_data,		\
-				  level, __FUNCTION__,			\
+				  level, __FILE__, __FUNCTION__,	\
 				  templ , ##args);			\
 } while (0)
 
+#define _vbi_vlog(hook, level, templ, ap)				\
+do {									\
+	_vbi_log_hook *_h = hook;					\
+									\
+	if ((NULL != _h && 0 != (_h->mask & level))			\
+	    || (_h = &_vbi_global_log, 0 != (_h->mask & level)))	\
+		_vbi_log_vprintf (_h->fn, _h->user_data,		\
+				  level, __FILE__, __FUNCTION__,	\
+				  templ, ap);				\
+} while (0)
+
 #define error(hook, templ, args...)					\
 	_vbi_log (hook, VBI_LOG_ERROR, templ , ##args)
 #define warning(hook, templ, args...)					\
@@ -312,15 +371,27 @@ do {									\
 #  define PRIx64 "llx"
 #endif
 
-/* Use this instead of strncpy(). strlcpy() is a BSD/GNU extension. */
+/* Should be defined in C99 limits.h? */
+#ifndef SIZE_MAX
+#  define SIZE_MAX ((size_t) -1)
+#endif
+
+/* __va_copy is a GNU extension. */
+#ifndef __va_copy
+#  define __va_copy(ap1, ap2) do { ap1 = ap2; } while (0)
+#endif
+
+/* Use this instead of strncpy(). strlcpy() is a BSD extension. */
 #ifndef HAVE_STRLCPY
 #  define strlcpy _vbi_strlcpy
 #endif
+#undef strncpy
+#define strncpy use_strlcpy_instead
 
 extern size_t
 _vbi_strlcpy			(char *			dst,
 				 const char *		src,
-				 size_t			len);
+				 size_t			size);
 
 /* strndup() is a BSD/GNU extension. */
 #ifndef HAVE_STRNDUP
@@ -351,4 +422,14 @@ _vbi_asprintf			(char **		dstp,
 				 const char *		templ,
 				 ...);
 
+#undef sprintf
+#define sprintf use_snprintf_or_asprintf_instead
+
 #endif /* MISC_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
