@@ -4,8 +4,9 @@
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: packet.c,v 1.25 2006/05/31 03:51:43 mschimek Exp $ */
+/* $Id: packet.c,v 1.28 2007/11/27 17:55:29 mschimek Exp $ */
 
 #include "site_def.h"
 
@@ -1090,7 +1091,7 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 			n->name[0] = 0;
 			unknown_cni(vbi, "VPS", cni);
 		} else {
-			strncpy((char *) n->name, name, sizeof(n->name) - 1);
+			strlcpy((char *) n->name, name, sizeof(n->name) - 1);
 			n->name[sizeof(n->name) - 1] = 0;
 		}
 
@@ -1146,7 +1147,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 					n->name[0] = 0;
 					unknown_cni(vbi, "8/30/1", cni);
 				} else {
-					strncpy((char *) n->name, name,
+					strlcpy((char *) n->name, name,
 						sizeof(n->name) - 1);
 					n->name[sizeof(n->name) - 1] = 0;
 				}
@@ -1236,7 +1237,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 					n->name[0] = 0;
 					unknown_cni(vbi, "8/30/2", cni);
 				} else {
-					strncpy((char *) n->name, name,
+					strlcpy((char *) n->name, name,
 						sizeof(n->name) - 1);
 					n->name[sizeof(n->name) - 1] = 0;
 				}
@@ -2559,3 +2560,10 @@ vbi_teletext_init(vbi_decoder *vbi)
 
 	vbi_teletext_channel_switched(vbi);     /* Reset */
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
