@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
  *  Description:
@@ -29,9 +29,31 @@
  *    Both UNIX domain and IPv4 and IPv6 sockets are implemented, but
  *    the latter ones are currently not officially supported.
  *
- *  $Id: proxy-msg.c,v 1.13 2004/12/30 02:25:29 mschimek Exp $
+ *  $Id: proxy-msg.c,v 1.19 2007/11/27 18:31:07 mschimek Exp $
  *
  *  $Log: proxy-msg.c,v $
+ *  Revision 1.19  2007/11/27 18:31:07  mschimek
+ *  Updated the FSF address in the copyright notice.
+ *
+ *  Revision 1.18  2007/08/27 10:17:50  mschimek
+ *  *** empty log message ***
+ *
+ *  Revision 1.17  2007/08/27 06:44:40  mschimek
+ *  vbi_proxy_msg_get_local_socket_addr, vbi_proxy_msg_accept_connection,
+ *  vbi_proxy_msg_resolve_symlinks: Replaced strncpy() by the faster a
+ *  safer strlcpy().
+ *  vbi_proxy_msg_logger, vbi_proxy_msg_accept_connection: Replaced
+ *  sprintf() by the safer snprintf().
+ *
+ *  Revision 1.16  2007/07/23 20:01:18  mschimek
+ *  *** empty log message ***
+ *
+ *  Revision 1.15  2006/05/22 09:08:46  mschimek
+ *  *** empty log message ***
+ *
+ *  Revision 1.14  2006/02/10 06:25:37  mschimek
+ *  *** empty log message ***
+ *
  *  Revision 1.13  2004/12/30 02:25:29  mschimek
  *  printf ptrdiff_t fixes.
  *
@@ -177,10 +199,10 @@ void vbi_proxy_msg_logger( int level, int clnt_fd, int errCode, const char * pTe
       memset(argv, 0, sizeof(argv));
       /* add pointer to file descriptor (for client requests) or pid (for general infos) */
       if (clnt_fd != -1)
-         sprintf(fdstr, "fd %d: ", clnt_fd);
+         snprintf(fdstr, sizeof (fdstr), "fd %d: ", clnt_fd);
       else
       {
-         sprintf(fdstr, "pid %d: ", (int)getpid());
+         snprintf(fdstr, sizeof (fdstr), "pid %d: ", (int)getpid());
       }
       argv[argc++] = fdstr;
 
@@ -485,7 +507,7 @@ vbi_bool vbi_proxy_msg_handle_read( VBIPROXY_MSG_STATE * pIO,
                pReadBuf->head.len  = pIO->readLen;
                pReadBuf->head.type = ntohl(pReadBuf->head.type);
                /* dprintf1("handle_io: fd %d: new block: size %d\n", pIO->sock_fd, pIO->readLen); */
-               if ((pIO->readLen > max_read_len) ||
+               if ((pIO->readLen > (size_t) max_read_len) ||
                    (pIO->readLen < sizeof(VBIPROXY_MSG_HEADER)))
                {
                   /* illegal message size -> protocol error */
@@ -506,7 +528,7 @@ vbi_bool vbi_proxy_msg_handle_read( VBIPROXY_MSG_STATE * pIO,
 
       if ((err == FALSE) && (pIO->readOff >= sizeof(VBIPROXY_MSG_HEADER)))
       {  /* in read phase two: read the complete message into the allocated buffer */
-         assert (pIO->readLen <= max_read_len);
+         assert (pIO->readLen <= (size_t) max_read_len);
 
          len = recv(pIO->sock_fd, (char*)pReadBuf + pIO->readOff,
                                   pIO->readLen - pIO->readOff, 0);
@@ -536,7 +558,8 @@ vbi_bool vbi_proxy_msg_handle_read( VBIPROXY_MSG_STATE * pIO,
          }
          else if ((len < 0) && (errno != EAGAIN) && (errno != EINTR))
          {  /* network error -> close the connection */
-            dprintf1("handle_io: read error on fd %d: len=%d, %s\n", pIO->sock_fd, len, strerror(errno));
+   	    dprintf1("handle_io: read error on fd %d: len=%ld, %s\n",
+		     pIO->sock_fd, (long) len, strerror(errno));
             result = FALSE;
          }
          else if (errno == EAGAIN)
@@ -754,7 +777,7 @@ static int vbi_proxy_msg_get_local_socket_addr( const char * pPathName, const st
       res->ai_addr      = (struct sockaddr *) saddr;
       res->ai_addrlen   = sizeof(struct sockaddr_un);
 
-      strncpy(saddr->sun_path, pPathName, sizeof(saddr->sun_path) - 1);
+      strlcpy(saddr->sun_path, pPathName, sizeof(saddr->sun_path) - 1);
       saddr->sun_path[sizeof(saddr->sun_path) - 1] = 0;
       saddr->sun_family = AF_UNIX;
 
@@ -920,11 +943,16 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
                hent = gethostbyaddr((void *) &peerAddr.sa, maxLength, AF_INET);
                if (hent != NULL)
                {
-                  strncpy(hname_buf, hent->h_name, sizeof(hname_buf) -1);
+                  strlcpy(hname_buf, hent->h_name, sizeof(hname_buf) -1);
                   hname_buf[sizeof(hname_buf) - 1] = 0;
                }
                else
-                  sprintf(hname_buf, "%s, port %d", inet_ntoa(((struct sockaddr_in *) &peerAddr.sa)->sin_addr), ((struct sockaddr_in *) &peerAddr.sa)->sin_port);
+	       {
+		  struct sockaddr_in *sa;
+
+		  sa = (struct sockaddr_in *) &peerAddr.sa;
+                  snprintf(hname_buf, sizeof (hname_buf), "%s, port %d", inet_ntoa(sa->sin_addr), sa->sin_port);
+	       }
 
                vbi_proxy_msg_logger(LOG_INFO, sock_fd, 0, "new connection from ", hname_buf, NULL);
                result = TRUE;
@@ -958,7 +986,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
             }
             else
             {  /* neither INET nor named socket -> internal error */
-               sprintf(hname_buf, "%d", peerAddr.sa.sa_family);
+               snprintf(hname_buf, sizeof (hname_buf), "%d", peerAddr.sa.sa_family);
                vbi_proxy_msg_logger(LOG_WARNING, -1, 0, "new connection via unexpected protocol family ", hname_buf, NULL);
             }
          }
@@ -969,7 +997,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
       }
       else
       {  /* socket address buffer too small: internal error */
-         sprintf(hname_buf, "need %d, have %d", length, maxLength);
+         snprintf(hname_buf, sizeof (hname_buf), "need %d, have %d", length, maxLength);
          vbi_proxy_msg_logger(LOG_WARNING, -1, 0, "new connection: saddr buffer too small: ", hname_buf, NULL);
       }
 
@@ -1012,7 +1040,7 @@ static char * vbi_proxy_msg_resolve_symlinks( const char * p_dev_name )
       if ((res == 0) && S_ISLNK(stbuf.st_mode))
       {
          name_len = readlink(p_path, link_name, sizeof(link_name));
-         if ((name_len > 0) && (name_len < sizeof(link_name)))
+         if ((name_len > 0) && (name_len < (int) sizeof(link_name)))
          {
             link_name[name_len] = 0;
             dprintf2("resolve_symlinks: following symlink %s to: %s\n", p_path, link_name);
@@ -1023,7 +1051,7 @@ static char * vbi_proxy_msg_resolve_symlinks( const char * p_dev_name )
                if (p_tmp2 != NULL)
                {  /* copy former path up to and including the separator character */
                   p_tmp2 += 1;
-                  strncpy(p_tmp, p_path, p_tmp2 - p_path);
+                  strlcpy(p_tmp, p_path, p_tmp2 - p_path);
                }
                else
                {  /* no path separator in the former path -> replace completely */
@@ -1194,14 +1222,14 @@ int vbi_proxy_msg_connect_to_server( vbi_bool use_tcp_ip, const char * pSrvHost,
          if (sock_fd == -1)
          {
             dprintf1("socket (ipv4): error %d, %s\n", errno, strerror(errno));
-            vbi_asprintf(ppErrorText, _("Cannot create socket: %s."),
+            asprintf(ppErrorText, _("Cannot create socket: %s."),
 			 strerror(errno));
          }
       }
       else
       {
          dprintf1("getaddrinfo (ipv4): %s\n", gai_strerror(rc));
-         vbi_asprintf(ppErrorText,
+         asprintf(ppErrorText,
 		      _("Invalid hostname or port: %s."),
 		      gai_strerror(rc));
       }
@@ -1222,9 +1250,9 @@ int vbi_proxy_msg_connect_to_server( vbi_bool use_tcp_ip, const char * pSrvHost,
          {
             dprintf1("connect: error %d, %s\n", errno, strerror(errno));
             if (use_tcp_ip)
-               vbi_asprintf(ppErrorText, _("Connection via TCP/IP failed, server not running or unreachable."));
+               asprintf(ppErrorText, _("Connection via TCP/IP failed, server not running or unreachable."));
             else
-               vbi_asprintf(ppErrorText, _("Connection via socket failed, server not running."));
+               asprintf(ppErrorText, _("Connection via socket failed, server not running."));
             close(sock_fd);
             sock_fd = -1;
          }
@@ -1233,7 +1261,7 @@ int vbi_proxy_msg_connect_to_server( vbi_bool use_tcp_ip, const char * pSrvHost,
       {
          dprintf1("fcntl (F_SETFL=O_NONBLOCK): error %d, %s\n",
 		  errno, strerror(errno));
-         vbi_asprintf(ppErrorText, _("Socket I/O error: %s."),
+         asprintf(ppErrorText, _("Socket I/O error: %s."),
 		      strerror(errno));
          close(sock_fd);
          sock_fd = -1;
@@ -1255,7 +1283,8 @@ int vbi_proxy_msg_connect_to_server( vbi_bool use_tcp_ip, const char * pSrvHost,
 vbi_bool vbi_proxy_msg_finish_connect( int sock_fd, char ** ppErrorText )
 {
    vbi_bool result = FALSE;
-   int  sockerr, sockerrlen;
+   int sockerr;
+   socklen_t sockerrlen;
 
    sockerrlen = sizeof(sockerr);
    if (getsockopt(sock_fd, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &sockerrlen) == 0)
@@ -1268,14 +1297,14 @@ vbi_bool vbi_proxy_msg_finish_connect( int sock_fd, char ** ppErrorText )
       else
       {  /* failed to establish a connection to the server */
          dprintf1("finish_connect: socket connect failed: %s\n", strerror(sockerr));
-         vbi_asprintf(ppErrorText, _("Cannot connect to server: %s."),
+         asprintf(ppErrorText, _("Cannot connect to server: %s."),
 		      strerror(sockerr));
       }
    }
    else
    {
       dprintf1("finish_connect: getsockopt: %s\n", strerror(errno));
-      vbi_asprintf(ppErrorText, _("Socket I/O error: %s."),
+      asprintf(ppErrorText, _("Socket I/O error: %s."),
 		   strerror(errno));
    }
 
@@ -1288,6 +1317,8 @@ vbi_bool vbi_proxy_msg_finish_connect( int sock_fd, char ** ppErrorText )
 static int
 vbi_proxy_msg_v4l_ioctl( int request, void * p_arg, vbi_bool * req_perm )
 {
+   p_arg = p_arg;
+
    switch (request)
    {
 #ifdef ENABLE_V4L
@@ -1421,3 +1452,10 @@ int vbi_proxy_msg_check_ioctl( VBI_DRIVER_API_REV vbi_api,
 }
 
 #endif  /* ENABLE_PROXY */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
