@@ -14,12 +14,21 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
- *  $Id: proxy-msg.h,v 1.9 2004/10/24 18:33:47 tomzo Exp $
+ *  $Id: proxy-msg.h,v 1.12 2007/11/27 18:31:07 mschimek Exp $
  *
  *  $Log: proxy-msg.h,v $
+ *  Revision 1.12  2007/11/27 18:31:07  mschimek
+ *  Updated the FSF address in the copyright notice.
+ *
+ *  Revision 1.11  2007/11/20 21:43:46  tomzo
+ *  Improvements and corrections in the proxy API documentation.
+ *
+ *  Revision 1.10  2007/07/23 20:01:18  mschimek
+ *  *** empty log message ***
+ *
  *  Revision 1.9  2004/10/24 18:33:47  tomzo
  *  - cleaned up socket I/O interface functions
  *  - added defines for norm change events
@@ -136,30 +145,43 @@ typedef enum
 
 /**
  * @ingroup Proxy
- * @brief Proxy scheduler configuration for background channel switching
+ * @brief Proxy scheduler parameters for background channel switching
+ *
+ * This structure is passed along with channel change requests for
+ * clients with priority @c VBI_CHN_PRIO_BACKGROUND.  The parameters
+ * are used by the proxy daemon to share channel control between
+ * multiple clients with background priority.
  */
 typedef struct
 {
         /**
-         * boolean: ignore struct unless TRUE
+         * Boolean: Ignore contents of this struct unless TRUE
          */
 	uint8_t			is_valid;
         /**
-         * Background prio: VPS/PDC, initial load, update, check, passive
+         * Sub-priority for channel scheduling at "background" priority.
+         * You can use aribtrary values in the range 0 ... 256, but as
+         * this value is only meaningful in relation to priorities used
+         * by other clients, you should stick to the scale defined by
+         * @ref VBI_CHN_SUBPRIO
          */
 	uint8_t			sub_prio;
         /**
-         * boolean: interruption allowed
+         * Boolean: Set to FALSE if your capture client needs an
+         * atomic time slice (i.e. would need to restart capturing
+         * from the beginning it it was interrupted.)
          */
 	uint8_t			allow_suspend;
 
 	uint8_t			reserved0;
         /**
-         * min. continuous use of that channel
+         * Minimum time slice your capture client requires. This value
+         * is used when multiple clients have the same sub-priority
+         * to give all clients channel control in a round-robin manner.
          */
 	time_t			min_duration;
         /**
-         * expected duration of use of that channel
+         * Expected duration of use of that channel
          */
 	time_t			exp_duration;
 
@@ -168,7 +190,7 @@ typedef struct
 
 /**
  * @ingroup Proxy
- * @brief General flags sent from clients to daemon during connect
+ * @brief General flags sent by the proxy daemon to clients during connect
  */
 typedef enum
 {
@@ -182,7 +204,7 @@ typedef enum
 
 /**
  * @ingroup Proxy
- * @brief General flags sent from daemon to clients during connect
+ * @brief General flags sent by clients to the proxy daemon during connect
  */
 typedef enum
 {
@@ -254,9 +276,25 @@ typedef enum
  */
 typedef enum
 {
+        /**
+         * Unknown device API - only used in error cases. Normally
+         * the proxy will always be aware of the driver API as it's
+         * determined by the type of capture context creation function
+         * used when the device is opened.
+         */
         VBI_API_UNKNOWN,
+        /**
+         * Video4Linux version 1 (i.e. Linux kernels 2.4 or older
+         * or old device drivers which have not been ported yet)
+         */
         VBI_API_V4L1,
+        /**
+         * Video4Linux version 2 (i.e. Linux kernels 2.6 and later)
+         */
         VBI_API_V4L2,
+        /**
+         * BSD Brooktree capture driver.
+         */
         VBI_API_BKTR
 } VBI_DRIVER_API_REV;
 
@@ -589,3 +627,10 @@ int      vbi_proxy_msg_check_ioctl( VBI_DRIVER_API_REV vbi_api,
                                     int request, void * p_arg, vbi_bool * req_perm );
 
 #endif  /* PROXY_MSG_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
