@@ -19,10 +19,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: search.c,v 1.12 2006/05/18 16:52:25 mschimek Exp $ */
+/* $Id: search.c,v 1.14 2007/11/27 18:31:07 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -634,3 +634,10 @@ vbi_search_delete(vbi_search *search)
 }
 
 #endif /* !HAVE_GLIBC21 && !HAVE_LIBUNICODE */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
