@@ -18,10 +18,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: search.h,v 1.2 2002/07/16 00:11:36 mschimek Exp $ */
+/* $Id: search.h,v 1.4 2007/11/27 18:31:07 mschimek Exp $ */
 
 #ifndef SEARCH_H
 #define SEARCH_H
@@ -87,3 +87,10 @@ extern vbi_search_status vbi_search_next(vbi_search *search, vbi_page **pg, int
 /* Private */
 
 #endif /* SEARCH_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
