@@ -4,8 +4,9 @@
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: teletext.c,v 1.23 2006/05/24 04:46:45 mschimek Exp $ */
+/* $Id: teletext.c,v 1.27 2007/11/27 17:54:49 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -2280,7 +2281,8 @@ vbi_format_vt_page(vbi_decoder *vbi,
 
 	/* Current page number in header */
 
-	sprintf(buf, "\2%x.%02x\7", vtp->pgno, vtp->subno & 0xff);
+	snprintf (buf, sizeof (buf),
+		  "\2%x.%02x\7", vtp->pgno, vtp->subno & 0xff);
 
 	/* Level 1 formatting */
 
@@ -2360,27 +2362,33 @@ vbi_format_vt_page(vbi_decoder *vbi,
 				break;
 			}
 
-			if (raw <= 0x1F)
+			if (raw <= 0x1F) {
 				ac.unicode = (hold & mosaic) ? held_mosaic_unicode : 0x0020;
-			else
+			} else {
 				if (mosaic && (raw & 0x20)) {
 					held_mosaic_unicode = mosaic_unicodes + raw - 0x20;
 					ac.unicode = held_mosaic_unicode;
 				} else
 					ac.unicode = vbi_teletext_unicode(font->G0,
 									  font->subset, raw);
+			}
 
-			if (!wide_char) {
+			if (wide_char) {
+				wide_char = FALSE;
+			} else {
 				acp[column] = ac;
 
 				wide_char = /*!!*/(ac.size & VBI_DOUBLE_WIDTH);
-
-				if (wide_char && column < (COLUMNS - 1)) {
-					acp[column + 1] = ac;
-					acp[column + 1].size = VBI_OVER_TOP;
+				if (wide_char) {
+                            		if (column < (COLUMNS - 1)) {
+                                    		acp[column + 1] = ac;
+                                    		acp[column + 1].size = VBI_OVER_TOP;
+					} else {
+                                    		acp[column].size = VBI_NORMAL_SIZE;
+						wide_char = FALSE;
+					}
 				}
-			} else
-				wide_char = FALSE;
+			}
 
 			/* set-after spacing attributes */
 
@@ -2636,3 +2644,10 @@ vbi_fetch_vt_page(vbi_decoder *vbi, vbi_page *pg,
 					  max_level, display_rows, navigation);
 	}
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
