@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: trigger.h,v 1.2 2002/10/22 04:42:40 mschimek Exp $ */
+/* $Id: trigger.h,v 1.4 2007/11/27 18:31:07 mschimek Exp $ */
 
 #ifndef TRIGGER_H
 #define TRIGGER_H
@@ -38,3 +38,10 @@ extern void		vbi_eacem_trigger(vbi_decoder *vbi, unsigned char *s);
 extern void		vbi_atvef_trigger(vbi_decoder *vbi, unsigned char *s);
 
 #endif /* TRIGGER_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
