@@ -19,10 +19,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: vbi.c,v 1.20 2006/05/25 08:09:35 mschimek Exp $ */
+/* $Id: vbi.c,v 1.22 2007/11/27 18:26:15 mschimek Exp $ */
 
 #include "site_def.h"
 
@@ -816,10 +816,18 @@ vbi_reset_prog_info(vbi_program_info *pi)
 void
 vbi_decoder_delete(vbi_decoder *vbi)
 {
+	struct event_handler *eh;
+
 	vbi_trigger_flush(vbi);
 
 	vbi_caption_destroy(vbi);
 
+	while (NULL != (eh = vbi->handlers)) {
+		vbi_event_handler_unregister (vbi,
+					      eh->handler,
+					      eh->user_data);
+	}
+
 	pthread_mutex_destroy(&vbi->prog_info_mutex);
 	pthread_mutex_destroy(&vbi->event_mutex);
 	pthread_mutex_destroy(&vbi->chswcd_mutex);
@@ -887,3 +895,10 @@ vbi_version			(unsigned int *		major,
 	if (minor) *minor = VBI_VERSION_MINOR;
 	if (micro) *micro = VBI_VERSION_MICRO;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
