@@ -18,10 +18,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: vt.h,v 1.8 2005/10/04 10:06:33 mschimek Exp $ */
+/* $Id: vt.h,v 1.11 2007/11/27 17:41:24 mschimek Exp $ */
 
 #ifndef VT_H
 #define VT_H
@@ -85,14 +85,14 @@ typedef enum {
  * Table 31: DRCS modes; Section 9.4.6, Table 9: Coding of Packet
  * X/28/3 for DRCS Downloading Pages.
  */
-typedef enum {
+enum drcs_mode {
 	DRCS_MODE_12_10_1,
 	DRCS_MODE_12_10_2,
 	DRCS_MODE_12_10_4,
 	DRCS_MODE_6_5_4,
 	DRCS_MODE_SUBSEQUENT_PTU = 14,
 	DRCS_MODE_NO_DATA
-} drcs_mode;
+};
 
 /*
     Only a minority of pages need this
@@ -134,7 +134,7 @@ typedef struct vt_triplet {
 	unsigned	address : 8;
 	unsigned	mode : 8;
 	unsigned	data : 8;
-} /* __attribute__ ((packed)) */ vt_triplet;
+} /* _vbi_attribute ((packed)) */ vt_triplet;
 
 typedef struct vt_pagenum {
 	unsigned	type : 8;
@@ -343,7 +343,7 @@ typedef struct {
 
 struct raw_page {
 	vt_page			page[1];
-        uint8_t		        drcs_mode[48];
+	uint8_t			drcs_mode[48];
 	int			num_triplets;
 	int			ait_page;
 };
@@ -446,3 +446,10 @@ extern vbi_bool		vbi_format_vt_page(vbi_decoder *, vbi_page *,
 					   int display_rows, vbi_bool navigation);
 
 #endif
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
