@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: xds_demux.h,v 1.6 2006/05/22 09:04:47 mschimek Exp $ */
+/* $Id: xds_demux.h,v 1.9 2007/11/27 17:40:53 mschimek Exp $ */
 
 #ifndef __ZVBI_XDS_DEMUX_H__
 #define __ZVBI_XDS_DEMUX_H__
@@ -26,6 +26,7 @@
 #include <inttypes.h>		/* uint8_t */
 #include <stdio.h>		/* FILE */
 #include "macros.h"
+#include "sliced.h"
 
 VBI_BEGIN_DECLS
 
@@ -148,10 +149,17 @@ typedef struct {
 	uint8_t			buffer[36];
 } vbi_xds_packet;
 
+/** @} */
+
 extern void
 _vbi_xds_packet_dump		(const vbi_xds_packet *	xp,
 				 FILE *			fp);
 
+/**
+ * @addtogroup XDSDemux
+ * @{
+ */
+
 /**
  * @brief XDS demultiplexer.
  *
@@ -182,23 +190,33 @@ vbi_xds_demux_reset		(vbi_xds_demux *	xd);
 extern vbi_bool
 vbi_xds_demux_feed		(vbi_xds_demux *	xd,
 				 const uint8_t		buffer[2]);
+extern vbi_bool
+vbi_xds_demux_feed_frame	(vbi_xds_demux *	xd,
+				 const vbi_sliced *	sliced,
+				 unsigned int		n_lines);
 extern void
 vbi_xds_demux_delete		(vbi_xds_demux *	xd);
 extern vbi_xds_demux *
 vbi_xds_demux_new		(vbi_xds_demux_cb *	callback,
 				 void *			user_data)
-  __attribute__ ((_vbi_alloc));
+  _vbi_attribute ((_vbi_alloc));
+
+/** @} */
 
 /* Private */
 
-/** @internal */
+/**
+ * @internal
+ */
 typedef struct {
 	uint8_t			buffer[32];
 	unsigned int		count;
 	unsigned int		checksum;
 } _vbi_xds_subpacket;
 
-/** @internal */
+/**
+ * @internal
+ */
 struct _vbi_xds_demux {
 	_vbi_xds_subpacket	subpacket[VBI_XDS_MAX_CLASSES]
 					 [VBI_XDS_MAX_SUBCLASSES];
@@ -217,8 +235,13 @@ _vbi_xds_demux_init		(vbi_xds_demux *	xd,
 				 vbi_xds_demux_cb *	callback,
 				 void *			user_data);
 
-/** @} */
-
 VBI_END_DECLS
 
 #endif /* __ZVBI_XDS_DEMUX_H__ */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
