@@ -15,10 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: explist.c,v 1.10 2006/02/10 06:25:38 mschimek Exp $ */
+/* $Id: explist.c,v 1.12 2007/11/27 18:26:48 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -322,7 +322,7 @@ static void
 dump_option_info(vbi_export *ex, vbi_option_info *oi)
 {
 	vbi_option_value val;
-	char *type_str;
+	const char *type_str;
 	int i;
 
 	switch (oi->type) {
