@@ -16,10 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: osc.c,v 1.29 2006/10/08 06:19:48 mschimek Exp $ */
+/* $Id: osc.c,v 1.34 2007/11/27 18:26:48 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -43,6 +43,8 @@
 #include "src/hamm.h"
 #include "src/io-sim.h"
 #include "src/raw_decoder.h"	/* _vbi_service_table[] */
+#include "src/proxy-msg.h"
+#include "src/proxy-client.h"
 
 #ifndef X_DISPLAY_MISSING
 
@@ -52,6 +54,7 @@
 
 vbi_capture *		cap;
 vbi_raw_decoder *	par;
+vbi_proxy_client *      pxc = NULL;
 int			src_w, src_h;
 vbi_sliced *		sliced;
 int			slines;
@@ -104,7 +107,8 @@ decode_ttx(uint8_t *buf, int line)
 	magazine = packet_address & 7;
 	packet = packet_address >> 3;
 
-        text += sprintf(text, "pg %x%02d ln %03d >", magazine, packet, line);
+        text += snprintf(text, 255,
+			 "pg %x%02d ln %03d >", magazine, packet, line);
 
         for (j = 0; j < 42; j++) {
 	   char c = _vbi_to_ascii (buf[j]);
@@ -133,17 +137,17 @@ dump_pil(int pil)
 	min = pil & 0x3F;
 
 	if (pil == PIL(0, 15, 31, 63))
-		sprintf(text, " PDC: Timer-control (no PDC)\n");
+		snprintf(text, 255, " PDC: Timer-control (no PDC)\n");
 	else if (pil == PIL(0, 15, 30, 63))
-		sprintf(text, " PDC: Recording inhibit/terminate\n");
+		snprintf(text, 255, " PDC: Recording inhibit/terminate\n");
 	else if (pil == PIL(0, 15, 29, 63))
-		sprintf(text, " PDC: Interruption\n");
+		snprintf(text, 255, " PDC: Interruption\n");
 	else if (pil == PIL(0, 15, 28, 63))
-		sprintf(text, " PDC: Continue\n");
+		snprintf(text, 255, " PDC: Continue\n");
 	else if (pil == PIL(31, 15, 31, 63))
-		sprintf(text, " PDC: No time\n");
+		snprintf(text, 255, " PDC: No time\n");
 	else
-		sprintf(text, " PDC: %05x, 200X-%02d-%02d %02d:%02d\n",
+		snprintf(text, 255, " PDC: %05x, 200X-%02d-%02d %02d:%02d\n",
 			pil, mon, day, hour, min);
         return text;
 }
@@ -161,7 +165,7 @@ decode_vps(uint8_t *buf)
         text_start=text=malloc(255);
         memset(text, 0, 255);
         
-	text += sprintf(text, "VPS: ");
+	text += snprintf(text, 255, "VPS: ");
 
 	c = vbi_rev8 (buf[1]);
 
@@ -177,7 +181,7 @@ decode_vps(uint8_t *buf)
 
 	l = (l + 1) % 16;
 
-	text += sprintf(text, " 3-10: %02x %02x %02x %02x %02x %02x %02x %02x (\"%s\")\n",
+	text += snprintf(text, 250, " 3-10: %02x %02x %02x %02x %02x %02x %02x %02x (\"%s\")\n",
 		buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6], buf[7], pr_label);
 
 	pcs = buf[2] >> 6;
@@ -191,9 +195,10 @@ decode_vps(uint8_t *buf)
 
 	pty = buf[12];
 
-	text += sprintf(text, " CNI: %04x PCS: %d PTY: %d ", cni, pcs, pty);
+	/* FIXME use real buffer size. */
+	text += snprintf(text, 100, " CNI: %04x PCS: %d PTY: %d ", cni, pcs, pty);
 
-	text += sprintf(text, " %s", dump_pil(pil));
+	text += snprintf(text, 50, " %s", dump_pil(pil));
    
         return(text_start);
 }
@@ -404,6 +409,7 @@ xevent(void)
 
 			case 'q':
 			case 'c':
+			case XK_Escape:
 				quit = TRUE;
 				break;
 
@@ -592,7 +598,7 @@ mainloop(void)
 		case 0: 
 			fprintf(stderr, "VBI read timeout%s\n",
 				ignore_error ? " (ignored)" : "");
-			if (ignore_error)
+			if (ignore_error || (pxc != NULL))
 				continue;
 			else
 				exit(EXIT_FAILURE);
@@ -610,7 +616,7 @@ mainloop(void)
 	}
 }
 
-static const char short_options[] = "123cd:enpsv";
+static const char short_options[] = "1234cd:enpsv";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
@@ -624,6 +630,7 @@ long_options[] = {
 	{ "v4l",	no_argument,		NULL,		'1' },
 	{ "v4l2-read",	no_argument,		NULL,		'2' },
 	{ "v4l2-mmap",	no_argument,		NULL,		'3' },
+	{ "proxy",	no_argument,		NULL,		'4' },
 	{ "verbose",	no_argument,		NULL,		'v' },
 	{ 0, 0, 0, 0 }
 };
@@ -654,6 +661,7 @@ main(int argc, char **argv)
 			/* fall through */
 		case '1':
 		case '3':
+		case '4':
 			interface = c - '0';
 			break;
 		case 'c':
@@ -695,7 +703,7 @@ main(int argc, char **argv)
 		assert ((par = vbi_capture_parameters(cap)));
 	} else {
 		do {
-			if (1 != interface) {
+			if ((2 == interface) || (3 == interface)) {
 				cap = vbi_capture_v4l2k_new
 					(dev_name, /* fd */ -1,
 					 /* buffers */ 5, &services,
@@ -744,6 +752,24 @@ main(int argc, char **argv)
 				free (errstr);
 			}
 
+			if (interface == 4) {
+                                pxc = vbi_proxy_client_create(dev_name, "capture", 0,
+                                                                &errstr, !!verbose);
+                                if (pxc != NULL) {
+                                        /* strip non-raw services, else request for raw is masked out */
+                                        unsigned int sv = services & (VBI_SLICED_VBI_525 |
+                                                                      VBI_SLICED_VBI_625);
+                                        cap = vbi_capture_proxy_new(pxc, 5, 0, &sv,
+                                                                    strict, &errstr );
+                                        if (cap)
+                                                break;
+
+                                        fprintf (stderr, "Cannot capture vbi data "
+                                                 "through proxy:\n%s\n", errstr);
+                                }
+				fprintf (stderr, "Cannot initialize proxy\n%s\n", errstr);
+                        }
+
 			/* BSD interface */
 			if (1) {
 				cap = vbi_capture_bktr_new (dev_name,
